/****************************************
Ez az llomny kizrolag az Effice Kft. (2083 Solymr Toldi u. 21.) tulajdona,
kizrolag csak az Effice Kft. engedlyvel, az engedlyben meghatrozott clra
s mdon hasznlhat fel. A program ingyenesen trtnt tadsra, mkdsrt 
az Effice Kft. semmilyen felelssget nem vllal.

Interface client for socket based communication.
Example program for generating orderentry record by watching orderbook.
****************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifndef WIN32
#  include <unistd.h>
#else 
#  include <wtypes.h>
#  include <winbase.h>
#endif

#include "ifsapi.h"
#include "ifsdefs.h"
#include "ifsutil.h"

#define ORDER_BUF_SIZE 1024     /*+ Order buffer length. +*/
#define SEP "|"                 /*+ Field separator character. +*/

static char Id[] = "gen_order ";

	/*+ trace flag +*/	
int trace;
	/*+ board id +*/
char * boardid;
	/*+ security id +*/
char * secid;	
	/*+ trdacc +*/
char * trdacc;
	/*+ by or sell +*/
int isbuy; 
	/*+ limit price +*/
int limit_price;
	/*+ max qty +*/
int max_qty;
        /* Prototypes */
static void gen_orders(IFSC_HDL handle, char *ob);
int main (int argc, char *argv[]);
static char * print_message(char * buf, char * msg, int len);
static void print_order(int price, int qty);
void usage( void);

#ifdef WIN32
#  define SLEEP(x) Sleep(x * 1000)
#else
#  define SLEEP(x) sleep(x)
#endif


/*+++++++++++++++++++++++++++++++++++++++++
Print a message.

char * print_message Points to the output string.

char * buf Pointer to the print buffer.

char * msg Pointer to message.

int len Length of the input.
+++++++++++++++++++++++++++++++++++++++++*/
static char * print_message(char * buf, char * msg, int len)
{
  int i;
  for (i = 0; i < len; i++){
    if (msg[i]) 
      buf[i] = msg[i];
    else
      buf[i] = '|'; 
  }
  buf[i]='\0';
  return buf;
}
/*+++++++++++++++++++++++++++++++++++++++++
Prints usage information.
+++++++++++++++++++++++++++++++++++++++++*/
void usage( void){
  fprintf(stderr, "Usage <boardid> <secid> <B/S> <price> <qty> <trdacc> [-c] [-t] [-hhost] [-sservice] [-uuser] [-ppwd] [-h]\n");
  fprintf(stderr, "  -c 	     continue\n");
  fprintf(stderr, "  -t 	     trace\n");
  fprintf(stderr, "  -hhost     host name of server\n");
  fprintf(stderr, "  -sservice  service name of server\n");
  fprintf(stderr, "  -uuser     user name\n");
  fprintf(stderr, "  -pwd       password\n");
  fprintf(stderr, "  -h         prints this information\n");
  fprintf(stderr, "If no host, service, user, pwd defined then \n");
  fprintf(stderr, "  value of IFSHOST, IFSSERVICE, IFSHOST, IFSPWD environment used\n");
  exit(1);    
}

/*+++++++++++++++++++++++++++++++++++++++++
Prints an order into stdout

int price Price of the order

int qty Quantity of the order
++++++++++++++++++++++++++++++++++++++++*/
static void print_order(int price, int qty)
{
  char * cp;
 
  fprintf(stdout, "%s|", trdacc);
  fprintf(stdout, "%c|", isbuy ? 'B' : 'S');
  fprintf(stdout, "%s|", "01000000000000000100000010000000");
  fprintf(stdout, "%s|", boardid);
  fprintf(stdout, "%s|", secid);
  fprintf(stdout, "%d|", price);
  fprintf(stdout, "%d|", 0);
  fprintf(stdout, "%d|", qty);
  fprintf(stdout, "%d|", 0);
  fprintf(stdout, "%s|", "test");
  fprintf(stdout, "%d|", 0);
  fprintf(stdout, "%d|", 0);
  fprintf(stdout, "%d|", 0);
  fprintf(stdout, "%d|", 0);
  fprintf(stdout, "%d|", 0);
  fprintf(stdout, "\n");
}

/*+++++++++++++++++++++++++++++++++++++++++
Parse orderbook, and generate order into stdout

IFSC_HDL handle Pointer to connection handler.

char * ob pointer to the orderbook data.
++++++++++++++++++++++++++++++++++++++++*/
static void gen_orders(IFSC_HDL handle, char *ob){
  int numbuy;
  int numsell;
  int i,p,q,n;

  ob += strlen(ob)+1; 
  if (*ob != 'Y') {
    fprintf(stderr, "The orderbook is not wathced!\n");
    ifsc_disconnect(handle);
    exit(1);
  }

  ob++;
  ob++;
  numbuy = atoi(ob);
  ob += strlen(ob)+1;
  numsell = atoi(ob); 
  ob += strlen(ob)+1;
  
  if (isbuy) {
    // skipping buys
    for(i=0; i<numbuy; i++) {
      ob += strlen(ob)+1;
      ob += strlen(ob)+1;
      ob += strlen(ob)+1;
      ob += strlen(ob)+1;
    }    
  }

  n = !isbuy ? numbuy : numsell;
      
  for(i=0; i<n; i++) 
  {
    // skip firm
    ob += strlen(ob)+1;
    p = atoi(ob);
    ob += strlen(ob)+1;
    q = atoi(ob);
    ob += strlen(ob)+1;
    // skip hidden
    ob += strlen(ob)+1;
      
    if ((!isbuy && p >= limit_price) ||
       (  isbuy && p <= limit_price))
    {
      if (q > max_qty) q = max_qty;
      max_qty -= q;
      print_order(p,q);
      if (max_qty <= 0) return;
    }
  }    
}


/*+++++++++++++++++++++++++++++++++++++++++
Socket based communication interface client. Clients can connect into this server to 
query orderbook information, and prints into stdout orderentry records while total quantity
has not reached the given level.

int main Tradition.

int argc Number of command line parameters.

char *argv[] Points to an array that contains the command line parameters.
+++++++++++++++++++++++++++++++++++++++++*/
int main (int argc, char *argv[])
{
  int i, j, l, len, s, tradeid, pid, mmts_type;
  char * recvb;
  int orderid;
  char * host = NULL;
  char * service = NULL;
  char * user = NULL;
  char * pwd = NULL;
  char * order_inp = NULL;
  FILE *f;
  char *buff;
  char secboardid[50];
  int cont = 0;
  IFSC_HDL handle;

  host = getenv("IFSHOST");
  service = getenv("IFSSERVICE");
  user = getenv("IFSUSER");
  pwd = getenv("IFSPWD");

  for (i = 1, j = 0; i < argc; i++){
    if (argv[i][0] == '-'){
      switch(argv[i][1]){
        case 't': 
          trace = 1; 
          break;
        case 'c': 
          cont = 1; 
          break;
        case 'h':
          host = argv[i]+2;
          break;
        case 's':
          service = argv[i]+2;
          break;
        case 'u':
          user = argv[i]+2;
          break;
        case 'p':
          pwd = argv[i]+2;
          break;
        case 'f':
          order_inp = argv[i]+2;
          break;
        case '?':
        case '-':
          usage();
          break;
      }
    } else {
      switch(j++){
        case 0:
          boardid = argv[i];
          break;
        case 1:
          secid = argv[i];
          break;  
        case 2:
          if (argv[i][0] != 'B' && argv[i][0] != 'S'){
            fprintf(stderr, "unknown buy/sell flag!\n");
            usage();
          }
          isbuy = argv[i][0] == 'B' ? 1 : 0; 
          break;
        case 3:
          limit_price = atoi(argv[i]);
          if (limit_price <= 0) {
            fprintf(stderr, "Invalid price %s\n", argv[i]);
            usage();
          }
          break;
        case 4:
          max_qty = atoi(argv[i]);
          if (max_qty < 1.0) {
            fprintf(stderr, "Invalid qty %s\n", argv[i]);
            usage();
          }
          break;
        case 5:
          trdacc = argv[i];
      }
    } 
  }

  if (trace) fprintf (stderr, "%s Compiled: %s %s\n", Id, __DATE__, __TIME__);

  if (!host) {
    fprintf(stderr, "The host is not defined!\n");
    usage();
  }

  if (!service) {
    fprintf(stderr, "The service is not defined!\n");
    usage();
  }

  if (!user) {
    fprintf(stderr, "The user is not defined!\n");
    usage();
  }

  if (!pwd) {
    fprintf(stderr, "The pwd is not defined!\n");
    usage();
  }
  
  if (!trdacc) {
    fprintf(stderr, "Argument missing!\n");
    usage();
  }

  if (ifsc_create(&handle, host, service)) {
    fprintf(stderr, "Error at creation - %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  if (trace)
    ifsc_set_trace(handle, IFS_SWITCH_ON);

  if (ifsc_connect(handle, user, pwd, &tradeid, &pid, &mmts_type)) {
    fprintf(stderr, "Error at connection - %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  secboardid[0]='0';
  strcpy(secboardid+1, boardid);
  l = strlen(secboardid);
  strcpy(secboardid + l, secid);
  s = ifsc_orderbook_conf(handle, secboardid, IFS_SWITCH_ON);
  if (s != 0 && s != IFS_OBALREADYWATCH){
    fprintf(stderr, "%s -- %s\n", ifsc_get_last_errmsg(handle), secboardid);
    ifsc_disconnect(handle);
    exit(1);
  }
  
  while(1) {
    if ((s = ifsc_get_next_orderbook(handle, &buff, &len, secboardid)) < IFS_OK){
      fprintf(stderr, "%s -- %s\n", ifsc_get_last_errmsg(handle), secboardid);
      ifsc_disconnect(handle);
      exit(1);
    }

    if (s != IFS_NOMORE) {

      gen_orders(handle, buff);
    
      if (max_qty <=0 || !cont) {
        ifsc_disconnect(handle);
        exit(0);
      }

      fflush(stdout);
      fflush(stderr);

    }
    SLEEP(1);
  }
    
  return ifsc_disconnect(handle);
}
