/****************************************
Ez az llomny kizrolag az Effice Kft. (2083 Solymr Toldi u. 21.) tulajdona,
kizrolag csak az Effice Kft. engedlyvel, az engedlyben meghatrozott clra
s mdon hasznlhat fel. A program ingyenesen trtnt tadsra, mkdsrt 
az Effice Kft. semmilyen felelssget nem vllal.

Interface client for socket based communication.
Example program for query data of K2.
****************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>

#ifdef WIN32
#  include <wtypes.h>
#  include <winbase.h>
#  include <stdlib.h>
#else
#  include <unistd.h>
#  include <stdlib.h>
#endif

#include "ifsapi.h"
#include "ifsdefs.h"
#include "ifsutil.h"

   /*+ Table info record. Will be filled from command line parameters. +*/
typedef struct 
   {
     int  flag;          /*+ if one then the table is watched else zero. +*/
     char name[12];      /*+ Name of the table. +*/
     char code[4];       /*+ Table code. The output record will be preceeded with this prefix. +*/
   } 
table_watch;

static char Id[] = "get_table ";

	/*+ trace flag +*/
static int trace;
	/*+ continuely scan +*/
static int cont;
	/*+ watched tables +*/
static table_watch tables[IFS_T_LAST] = {{0,"",           ""},    /*  Skip zeroth record. */ 
                                         {0,"market",     "MKT"},
                                         {0,"instr",      "INS"},
                                         {0,"sector",     "SCT"},
                                         {0,"board",      "BRD"},
                                         {0,"secboard",   "SBD"},
                                         {0,"firm",       "FRM"},
                                         {0,"user",       "USR"},
                                         {0,"order",      "ORD"},
                                         {0,"trade",      "TRD"},
                                         {0,"negdeal",    "NGD"},
                                         {0,"orderentry", "OEN"},
                                         {0,"mmorder",    "MMO"},
                                         {0,"auditevent", "AUD"},
									{ 0,"priceparam", "PRI"}};

	/* Prototypes */
int main (int argc, char *argv[]);
static char * print_message(char * buf, char * msg, int len, char mode , int mmts_type);
void usage( void);

#ifdef WIN32
#  define SLEEP(x) Sleep(x * 1000)
#else
#  define SLEEP(x) sleep(x)
#endif

/*+++++++++++++++++++++++++++++++++++++++++
Print a message.

char * print_message Points to the message.

char * buf Pointer to the print buffer(output).

char * msg Pointer to message (input).

int len Length of the input.

char mode Mode of the query (g|i|q)

Prints K2 data in msg into buf, fields separeted by | char.
If mode is g (field change), then field_code value pair printed into buf.
+++++++++++++++++++++++++++++++++++++++++*/
static char * print_message(char * buf, char * msg, int len, char mode , int mmts_type)
{
  int i, pos =0, data_len;
  char field_code, *data;

  if (mode != 'g') {
    for (i = 0; i < len; i++){
      if (msg[i]) 
        buf[i] = msg[i];
      else
         buf[i] = '|'; 
    }
    buf[i]='\0';
  } 
  else {
    sprintf(buf, "%s|", msg); /* record id */
    for (i = IFS_OK; i == IFS_OK;) { 
	if ( mmts_type == MMTS_TYPE_1)
		i = ifs_get_next_field(msg, len, &pos, &field_code, &data, &data_len);
	else 
		i = ifs_get_next_field_mmts2(msg, len, &pos, &field_code, &data, &data_len);
      if (i == IFS_OK) {
        sprintf(buf + strlen(buf), " %d:%s|", field_code, data); /* field codes and field data */
      }
    }        
  }

  return buf;
}

/*+++++++++++++++++++++++++++++++++++++++++
Prints usage information.
+++++++++++++++++++++++++++++++++++++++++*/
void usage( void){
  fprintf(stderr, "Usage <-1|-2> { all | <table name> [<table name> ...] } [-c] [-t] [q|i|g] [-hhost] [-sservice] [-uuser] [-ppwd]\n");
  fprintf(stderr, "  -1         connecting to MMTS1 server\n");
  fprintf(stderr, "  -2         connecting to MMTS2 server\n");
  fprintf(stderr, "             if both -1 and -2 given then using multi session!\n");
  fprintf(stderr, "  -c         continue\n");
  fprintf(stderr, "  -t         trace\n");
  fprintf(stderr, "  -q         seqno mode (default) \n");
  fprintf(stderr, "  -i         index mode\n");
  fprintf(stderr, "  -g         field change mode\n");
  fprintf(stderr, "  -hhost     host name of server\n");
  fprintf(stderr, "  -sservice  service name of server\n");
  fprintf(stderr, "  -uuser     user name for server\n");
  fprintf(stderr, "  -ppwd      password for server\n");
  fprintf(stderr, "             if both -1 and -2 given -h, -s, -u, -p valid only for MMTS1\n");
  fprintf(stderr, "If no host, service, user, pwd defined then \n");
  fprintf(stderr, "  using value of IFSHOST, IFSSERVICE, IFSUSER, IFSPWD for MMTS1 or \n");
  fprintf(stderr, "                 IFS2HOST, IFS2SERVICE, IFS2USER, IFS2PWD for MMTS2\n");
  fprintf(stderr, "  If -2 given and IFS2* is not defined using IFS* parameters\n");
  fprintf(stderr, "  If both -1 and -2 IFS2SERVICE should be defined!\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "table_names: market\n"); 
  fprintf(stderr, "             instr\n"); 
  fprintf(stderr, "             sector\n"); 
  fprintf(stderr, "             board\n"); 
  fprintf(stderr, "             secboard\n"); 
  fprintf(stderr, "             firm\n");
  fprintf(stderr, "             user\n");
  fprintf(stderr, "             order\n");
  fprintf(stderr, "             trade\n");
  fprintf(stderr, "             negdeal\n");
  fprintf(stderr, "             orderentry\n");
  fprintf(stderr, "             mmorder\n");
  fprintf(stderr, "             auditevent\n");
  fprintf(stderr, "             priceparam\n");
  fprintf(stderr, "             systime\n");
  exit(1);    
}


/*+++++++++++++++++++++++++++++++++++++++++
Query one changed record.

int get_next_data Returns status of ifsc function calls.

IFSC_HDL handle Pointer to connection handler.

char ** recvb Return a pointer to the receiver buffer.

int * len Return len of the message in the *recvb buffer.

int i Table code.

int * seqno Returns last seqno of the record of message.

char mode Mode of the query (q|i|g) see usage. 
+++++++++++++++++++++++++++++++++++++++++*/
int get_next_data(IFSC_HDL handle, char ** recvb, int * len, int i, int * seqno, char mode)
{
  int s = 0;
  switch (mode) {
    case 'q':
      s = ifsc_get_next_record(handle, recvb, len, i, seqno);
      break;
    case 'i':
      s = ifsc_get_next_idx(handle, recvb, len, i, seqno);
      break;
    case 'g':
      s = ifsc_get_next_field_chg(handle, recvb, len, i, seqno); 
      break;
  }
  return s;
}
/*+++++++++++++++++++++++++++++++++++++++++
Socket based communication interface client. Clients can connect into this server to 
query changed or new record of K2.

int main Tradition.

int argc Number of command line parameters.

char *argv[] Points to an array that contains the command line parameters.
+++++++++++++++++++++++++++++++++++++++++*/
int main (int argc, char *argv[])
{
  int s, i, ret=0, len, istable = 0, j, seqno, tradeid, pid, systime = 0, mmts_type;
  char * recvb;
  char ptr_buf[0x8000];
  char * host = NULL;
  char * service = NULL;
  char * user = NULL;
  char * pwd = NULL;
  char * host2 = NULL;
  char * service2 = NULL;
  char * user2 = NULL;
  char * pwd2 = NULL;
  char mode = 'q';
  int session1 = 0, session2 = 0, nextsession = 1; 
  int tedate, tetime, k2date, k2time, pgwqueryoffset, pgwstate;
  struct tm *timevartm;
  static char timevarbuf[16];
  time_t timevar;

  IFSC_HDL handle, handle1, handle2;

  host = getenv("IFSHOST");
  service = getenv("IFSSERVICE");
  user = getenv("IFSUSER");
  pwd = getenv("IFSPWD");
  host2 = getenv("IFS2HOST");
  service2 = getenv("IFS2SERVICE");
  user2 = getenv("IFS2USER");
  pwd2 = getenv("IFS2PWD");


  for (i = 1; i < argc; i++){
    if (argv[i][0] == '-'){
      switch(argv[i][1]){
        case 't': 
          trace = 1; 
          break;
        case 'h':
          host = argv[i]+2;
          break;
        case 's':
          service = argv[i]+2;
          break;
        case 'u':
          user = argv[i]+2;
          break;
        case 'p':
          pwd = argv[i]+2;
          break;
        case 'c':
          cont = 1;
          break;
        case 'i':
          mode = 'i';
          break; 
        case 'q':
          mode = 'q';
          break; 
        case 'g':
          mode = 'g';
          break; 
        case '1':
          session1 = 1;
          break;
        case '2':
          session2 = 1;
          break;
      }
    } else {
      if (!strcmp(argv[i], "market")){
        tables[IFS_T_MARKET].flag = 1;
      }  
      else if (!strcmp(argv[i], "instr")){
        tables[IFS_T_INSTR].flag = 1;
      }  
      else if (!strcmp(argv[i], "sector")){
        tables[IFS_T_SECTOR].flag = 1;
      }  
      else if (!strcmp(argv[i], "board")){
        tables[IFS_T_BOARD].flag = 1;
      }  
      else if (!strcmp(argv[i], "secboard")){
        tables[IFS_T_SECBOARD].flag = 1;
      }  
      else if (!strcmp(argv[i], "firm")){
        tables[IFS_T_FIRM].flag = 1;
      }  
      else if (!strcmp(argv[i], "user")){
        tables[IFS_T_USER].flag = 1;
      }  
      else if (!strcmp(argv[i], "order")){
        tables[IFS_T_ORDER].flag = 1;
      }  
      else if (!strcmp(argv[i], "trade")){
        tables[IFS_T_TRADE].flag = 1;
      }  
      else if (!strcmp(argv[i], "negdeal")){
        tables[IFS_T_NEGDEAL].flag = 1;
      }  
      else if (!strcmp(argv[i], "orderentry")){
        tables[IFS_T_ORDERENTRY].flag = 1;
      }
      else if (!strcmp(argv[i], "mmorder")){
        tables[IFS_T_MMORDER].flag = 1;
      }
      else if (!strcmp(argv[i], "auditevent")){
        tables[IFS_T_AUDITEVENT].flag = 1;
	  }
	  else if (!strcmp(argv[i], "priceparam")){
        tables[IFS_T_PRICEPARAM].flag = 1;
	  }
      else if (!strcmp(argv[i], "all")){
        for (j=1; j< IFS_T_LAST; j++)
          tables[j].flag = 1;
        systime = 1;
      }
      else if (!strcmp(argv[i], "systime")){
        systime = 1;
      }
      else {
        fprintf(stderr, "Unkown table name!\n");
        usage();
      }  
      istable = 1;
    }
  }

  if (trace) fprintf (stderr, "%s Compiled: %s %s\n", Id, __DATE__, __TIME__);

  if (!istable) {
    fprintf(stderr, "Not enough parameter: table is not defined!\n");
    usage();
  }

  if (!session1 && !session2) {
    fprintf(stderr, "MMTS version -1 or -2 should be defined!\n");
    usage();
  }

  if (!host) {
    fprintf(stderr, "The host is not defined!\n");
    usage();
  }

  if (!service) {
    fprintf(stderr, "The service is not defined!\n");
    usage();
  }

  if (!user) {
    fprintf(stderr, "The user is not defined!\n");
    usage();
  }

  if (!pwd) {
    fprintf(stderr, "The pwd is not defined!\n");
    usage();
  }

  if (session2) {
    if (!service2) {
      if (session1) {
        fprintf(stderr, "In case multisession (-1 and -2 given) the IFS2SERVICE should be defined!\n");
        usage();
      }
      service2 = service;
    }
    if (!host2) host2 = host;
    if (!user2) user2 = user;
    if (!pwd2) pwd2 = pwd;
  }

  if (session1) {
    if (ifsc_create(&handle1, host, service)) {
      fprintf(stderr, "Error at creation: %s\n", ifsc_get_last_errmsg(handle1));
      exit(1);
    }

    if (trace) {
      if (ifsc_set_trace(handle1, IFS_SWITCH_ON)) {
        fprintf(stderr, "Error at set_trace: %s\n", ifsc_get_last_errmsg(handle1));
      }
    }
    
    if (ifsc_connect(handle1, user, pwd, &tradeid, &pid, &mmts_type)) {
      fprintf(stderr, "Error at connection: %s\n", ifsc_get_last_errmsg(handle1));
      exit(1);
    }

    if (mmts_type != MMTS_TYPE_1) {
      fprintf(stderr, "Error: MMTS type of server connected to IFSSERVICE is not 1!");
      exit(1);
    }
  }

  if (session2) {
    if (ifsc_create(&handle2, host2, service2)) {
      fprintf(stderr, "Error at creation 2nd: %s\n", ifsc_get_last_errmsg(handle2));
      exit(1);
    }

    if (trace) {
      if (ifsc_set_trace(handle2, IFS_SWITCH_ON)) {
        fprintf(stderr, "Error at set_trace: %s\n", ifsc_get_last_errmsg(handle2));
      }
    }
    
    if (ifsc_connect(handle2, user2, pwd2, &tradeid, &pid, &mmts_type)) {
     fprintf(stderr, "Error at connection 2nd: %s\n", ifsc_get_last_errmsg(handle2));
      exit(1);
    } 

    if (mmts_type != MMTS_TYPE_2) {
      fprintf(stderr, "Error: MMTS type of server connected to IFSSERVICE is not 2!");
      exit(1);
    }
  }

  handle = session1 ? handle1 : handle2;

  while(1) {
    if (session1 && session2) {
      switch (nextsession) {
        case 1: 
          handle = handle1;
          nextsession = 2;
          break;
        case 2: 
          handle = handle2;
          nextsession = 1;
          break;
        default:
          fprintf(stderr, "Internal Error : invalid nextsession parameter\n");    
      }
    }

    if (systime) {
       if ((s = ifsc_get_systime(handle, &recvb, &len)) == IFS_OK) {
         if (trace) fprintf(stderr, "systime request: status: %d, len %d\n", s, len);
         len = ifs_get_int(&tedate, recvb);
         len += ifs_get_int(&tetime, recvb + len);
         len += ifs_get_int(&k2date, recvb + len);
         len += ifs_get_int(&k2time, recvb + len);
         len += ifs_get_int(&pgwqueryoffset, recvb + len);
         len += ifs_get_int(&pgwstate, recvb + len);
		 timevar = time(NULL);
         timevartm = localtime(&timevar);
         strftime(timevarbuf, 16, "%Y%m%d|%H%M%S", timevartm);
         fprintf(stdout, "%c|STM|%d|%06d|%d|%06d|%04d|%02d|%s\n", handle == handle1 ? '1' : '2', tedate, tetime, k2date, k2time, pgwqueryoffset, pgwstate, timevarbuf);
       } else {
         fprintf(stderr, "Error at getting systime: %s\n", ifsc_get_last_errmsg(handle));
      }
    }

    for (i = 1; i < IFS_T_LAST; i++){
      if (!tables[i].flag) continue;
      while ((s = get_next_data(handle, &recvb, &len, i, &seqno, mode)) == 0) {
        if (trace) fprintf(stderr, " %s: status: %d, len %d, seqno: %d\n", tables[i].name ,s, len, seqno);
        fprintf(stdout, "%c|%s|%s\n", handle == handle1 ? '1' : '2', tables[i].code, print_message(ptr_buf, recvb, len, mode, mmts_type));
        fflush(stdout);
      }
      if (s < 0) {
        fprintf(stderr, "Error at getting data:  %s\n", ifsc_get_last_errmsg(handle));
      }
    }

    if (!cont && nextsession == 1) {
      if (session1 && ifsc_disconnect(handle1)) {
        fprintf(stderr, "Error at disconnect: %s\n", ifsc_get_last_errmsg(handle1));
        exit(1);
      }
      if (session2 && ifsc_disconnect(handle2)) {
        fprintf(stderr, "Error at disconnect: %s\n", ifsc_get_last_errmsg(handle2));
        exit(1);
      }
      exit(0);
    }
    
    if (nextsession == 1) SLEEP(1);
  } 

  return 0;
}
