/****************************************
Ez az llomny kizrolag az Effice Kft. (2083 Solymr Toldi u. 21.) tulajdona,
kizrolag csak az Effice Kft. engedlyvel, az engedlyben meghatrozott clra
s mdon hasznlhat fel. A program ingyenesen trtnt tadsra, mkdsrt 
az Effice Kft. semmilyen felelssget nem vllal.

Interface client for socket based communication.
Example program for querying and setting orderbook watches.
****************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef WIN32
#  include <unistd.h>
#endif


#include "ifsapi.h"
#include "ifsdefs.h"
#include "ifsutil.h"

static char Id[] = "ob_watch ";

      /* Protypes */
void usage( void);
int main (int argc, char *argv[]);

/*+++++++++++++++++++++++++++++++++++++++++
Prints usage information.
+++++++++++++++++++++++++++++++++++++++++*/
void usage( void){
  fprintf(stderr, "Usage <secboardid| -l> [-r] [-m] [-t] [-hhost] [-sservice] [-uuser] [-ppwd]\n");
  fprintf(stderr, "  -l 	list watched orderbooks\n");
  fprintf(stderr, "  -r 	remove orderbook\n");
  fprintf(stderr, "  -m         market by price configuring/listing\n");
  fprintf(stderr, "  -t 	trace\n");
  fprintf(stderr, "  -hhost     host name of server\n");
  fprintf(stderr, "  -sservice  service name of server\n");
  fprintf(stderr, "  -uuser     user name\n");
  fprintf(stderr, "  -ppwd      password\n");
  fprintf(stderr, "If no host, service, user, pwd defined then \n");
  fprintf(stderr, "  value of IFSHOST, IFSSERVICE, IFSHOST, IFSPWD environment used\n");
  exit(1);    
}

/*+++++++++++++++++++++++++++++++++++++++++
Socket based communication interface client. Clients can connect into this server to 
query or set orderbook and market by price watches.

int main Tradition.

int argc Number of command line parameters.

char *argv[] Points to an array that contains the command line parameters.
+++++++++++++++++++++++++++++++++++++++++*/
int main (int argc, char *argv[])
{
  int s, i, len, tradeid, pid, mmts_type;
  int trace = 0;
  int remove = 0;
  int list = 0;
  int marketbyprx = 0;
  char * secboardid = NULL;
  char * host = NULL;
  char * service = NULL;
  char * user = NULL;
  char * pwd = NULL;
  char * buffer;
  IFSC_HDL handle;

  host = getenv("IFSHOST");
  service = getenv("IFSSERVICE");
  user = getenv("IFSUSER");
  pwd = getenv("IFSPWD");


  for (i = 1; i < argc; i++){
    if (argv[i][0] == '-'){
      switch(argv[i][1]){
        case 'r':
          remove = 1;
          break;
        case 'm':
          marketbyprx = 1;
          break;
        case 't': 
          trace = 1; 
          break;
        case 'h':
          host = argv[i]+2;
          break;
        case 's':
          service = argv[i]+2;
          break;
        case 'u':
          user = argv[i]+2;
          break;
        case 'p':
          pwd = argv[i]+2;
          break;
        case 'l':
          list = 1;
          break;
      }
    } else {
      secboardid = argv[i];
    }
  }

  if (trace) fprintf (stderr, "%s Compiled: %s %s\n", Id, __DATE__, __TIME__);

  if (!secboardid && !list) {
    fprintf(stderr, "Not enough parameter!\n");
    usage();
  }

  if (!host) {
    fprintf(stderr, "The host is not defined!\n");
    usage();
  }

  if (!service) {
    fprintf(stderr, "The service is not defined!\n");
    usage();
  }

  if (!user) {
    fprintf(stderr, "The user is not defined!\n");
    usage();
  }

  if (!pwd) {
    fprintf(stderr, "The pwd is not defined!\n");
    usage();
  }

  if (ifsc_create(&handle, host, service)) {
    fprintf(stderr, "Error at creation: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  if (trace)
    ifsc_set_trace(handle, IFS_SWITCH_ON);

  if (ifsc_connect(handle, user, pwd, &tradeid, &pid, &mmts_type)) {
    fprintf(stderr, "Error at connection: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  if (list) {

    if (!marketbyprx)
      s = ifsc_get_orderbook_list(handle, &buffer, &len);
    else 
      s = ifsc_get_marketbyprx_list(handle, &buffer, &len);

    if (s == IFS_OK) { 
      for(i=0; i < len; i++)
        if (buffer[i] == 0) 
          buffer[i] = '\n';
      write(1, buffer, len);
      
    } else {
      fprintf (stderr, "%s -- list\n", ifsc_get_last_errmsg(handle));
    }

  } else {

    if (!marketbyprx)
      s = ifsc_orderbook_conf(handle, secboardid, (remove ? IFS_SWITCH_OFF : IFS_SWITCH_ON));
    else 
      s = ifsc_marketbyprx_conf(handle, secboardid, (remove ? IFS_SWITCH_OFF : IFS_SWITCH_ON));

    if (s == IFS_OK) {
      if (trace) fprintf (stderr, " orderbook watch %s for %s\n", 
        (remove ? "removed" : "added"), secboardid);
    }  
    else {
      fprintf (stderr, "%s -- %s\n", ifsc_get_last_errmsg(handle), secboardid);
    }
  }     

  return ifsc_disconnect(handle);
}
