/****************************************
Ez az llomny kizrolag az Effice Kft. (2083 Solymr Toldi u. 21.) tulajdona,
kizrolag csak az Effice Kft. engedlyvel, az engedlyben meghatrozott clra
s mdon hasznlhat fel. A program ingyenesen trtnt tadsra, mkdsrt 
az Effice Kft. semmilyen felelssget nem vllal.

Interface client for socket based communication.
Example program for sending an orderentry.
****************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#ifndef WIN32
#  include <unistd.h>
#else 
#  include <wtypes.h>
#  include <winbase.h>
#  include <io.h>
#endif


#include "ifsapi.h"
#include "ifsdefs.h"
#include "ifsutil.h"

#define ORDER_BUF_SIZE 1024     /*+ Maximal size of the order buffer. +*/
#define SEP "|"                 /*+ Field separator character. +*/

/* -------- orderentry types -- */
#define IFS_OE_ENTRY            'E'     /*+ Orderentry is an entry (new order). +*/
#define IFS_OE_AMEND            'A'     /*+ Orderentry is an order amendment +*/
#define IFS_OE_WITHDRAW         'W'     /*+ Orderentry is an order(s) withdrawal +*/
#define IFS_OE_TICKUPTICKDOWN   'T'     /*+ Orderentry is an tickuptickdown +*/

static char Id[] = "send_order ";

	/*+ trace flag +*/	
static int trace;

        /*+ System defined variable. +*/
extern int errno;
        /* Prototypes */
int main (int argc, char *argv[]);
static void order_mmts1(IFSC_HDL handle, char *line, int line_num, char entrytype);
static void order_mmts2(IFSC_HDL handle, char *line, int line_num, char entrytype);
static char * print_message(char * buf, char * msg, int len);
void usage( void);

/*+++++++++++++++++++++++++++++++++++++++++
Print a message.

static char * print_message Points to the output string.

char * buf Pointer to the print buffer.

char * msg Pointer to message.

int len Length of the input.
+++++++++++++++++++++++++++++++++++++++++*/
static char * print_message(char * buf, char * msg, int len)
{
  int i;
  for (i = 0; i < len; i++){
    if (msg[i]) 
      buf[i] = msg[i];
    else
      buf[i] = '|'; 
  }
  buf[i]='\0';
  return buf;
}
/*+++++++++++++++++++++++++++++++++++++++++
Prints usage information.
+++++++++++++++++++++++++++++++++++++++++*/
void usage( void){
  fprintf(stderr, "Usage -e<E/A/W/T> <-1|-2> [-forders] [-t] [-hhost] [-sservice] [-uuser] [-ppwd] [-h]\n");
  fprintf(stderr, "  -e<E/A/W/T>  entries, amends, withdrawns ot tickuptickdown\n");
  fprintf(stderr, "  <-1|-2>    using fields of MMTS1 or MMTS2 system\n");
  fprintf(stderr, "  -forders   name of the file of input orders, if not given program reads from the stdin\n");
  fprintf(stderr, "  -t 	     trace\n");
  fprintf(stderr, "  -hhost     host name of server\n");
  fprintf(stderr, "  -sservice  service name of server\n");
  fprintf(stderr, "  -uuser     user name\n");
  fprintf(stderr, "  -pwd       password\n");
  fprintf(stderr, "  -h         prints this information\n");
  fprintf(stderr, "If no host, service, user, pwd defined then \n");
  fprintf(stderr, "  value of IFSHOST, IFSSERVICE, IFSHOST, IFSPWD environment used\n");
  exit(1);    
}

#define FN_ENTRY_MMTS1        15      /*+ Number of fields in case of entry. +*/
#define FN_AMEND_MMTS1        13      /*+ Number of fields in case of amendment. +*/
#define FN_WITHDRAW_MMTS1     13      /*+ Number of fields in case of withdrawal. +*/
#define FN_TICK_MMTS1         9       /*+ Number of fields in case of tickuptickdown. +*/

#define FN_ENTRY_MMTS2        22      /*+ Number of fields in case of entry. +*/
#define FN_AMEND_MMTS2        18      /*+ Number of fields in case of amendment. +*/
#define FN_WITHDRAW_MMTS2     20      /*+ Number of fields in case of withdrawal. +*/
#define FN_TICK_MMTS2         9       /*+ Number of fields in case of tickuptickdown. +*/

/*+++++++++++++++++++++++++++++++++++++++++
The parse_line function looks for the identifier in the options structure and writes the value into its variable.
Using MMTS1 fields.

IFSC_HDL Connection handle.

char *line Single line of the config file. The line should look like: identifier = value.

int line_num Line number of the input file.

char entrytype Entry, amend or withdraw.

This function takes a line and splits into fields of orders, then send that into pgw.
+++++++++++++++++++++++++++++++++++++++++*/
static void order_mmts1(IFSC_HDL handle, char *line, int line_num, char entrytype)
{
  char * cp;
  int i, len, s, orderid;
  char ptr_buf[0x8000];
  char buf[1024];
  int fieldnum;
  
  switch (entrytype) {
    case IFS_OE_ENTRY:
      fieldnum = FN_ENTRY_MMTS1;
      break;
    case IFS_OE_AMEND:
      fieldnum = FN_AMEND_MMTS1;
      break;
    case IFS_OE_WITHDRAW:
      fieldnum = FN_WITHDRAW_MMTS1 ;
      break;
    case IFS_OE_TICKUPTICKDOWN:
      fieldnum = FN_TICK_MMTS1 ;
      break;
    default:
      fprintf (stderr, "Bad entry type %c\n", entrytype);
      return;
  }
  // count number of fields
  cp = strchr(line, *SEP);
  for (i=1; i<fieldnum && cp && *cp+1; i++, cp++)
    cp = strchr(cp, *SEP);

  if (i<fieldnum){
    fprintf (stderr, "Missing field from line %d\n", line_num);
    return;
  }

  switch (entrytype) {
    case IFS_OE_ENTRY:
  len = ifs_set_ids(buf, strtok(line, SEP));
  len += ifs_set_char(buf + len, *strtok(NULL, SEP));
  len += ifs_set_string(buf + len, IFS_ORDERMETHOD_LEN, strtok(NULL, SEP));
  len += ifs_set_string(buf + len, IFS_BOARDID_LEN, strtok(NULL, SEP));
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));
      break;
    case IFS_OE_AMEND:
  len = ifs_set_int(buf, atoi(strtok(line, SEP)));
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));
  len += ifs_set_string(buf + len, IFS_ORDERMETHOD_LEN, strtok(NULL, SEP));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));
      break;
    case IFS_OE_WITHDRAW:
  len = ifs_set_int(buf, atoi(strtok(line, SEP)));
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));
  len += ifs_set_char(buf + len, *strtok(NULL, SEP));
  len += ifs_set_string(buf + len, IFS_BOARDID_LEN, strtok(NULL, SEP));
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));
  len += ifs_set_char(buf + len, *strtok(NULL, SEP));
  len += ifs_set_char(buf + len, *strtok(NULL, SEP));
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));
      break;
    case IFS_OE_TICKUPTICKDOWN:
  len = ifs_set_ids(buf, strtok(line, SEP));                                // trdacc
  len += ifs_set_char(buf + len, *strtok(NULL, SEP));                       // buysell
  len += ifs_set_string(buf + len, IFS_BOARDID_LEN, strtok(NULL, SEP));     // board
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));                         // sec
  len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // price
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));                         // user
  len += ifs_set_ids(buf + len, strtok(NULL, SEP));                         // firm
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));   // brokerref
  len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));   // inernalref
      break;
    default:
      fprintf (stderr, "Bad entry type %c\n", entrytype);
      return;
  }

  if (trace) {
    print_message(ptr_buf, buf, len);
    fprintf(stderr, ">%s<\n",ptr_buf);
  }  

  switch (entrytype) {
    case IFS_OE_ENTRY:
      if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_ADD, buf, len, &orderid)) == 0) {
        if (trace) fprintf (stderr, " order_add (orderid %d) sent\n", orderid);
      } 
      else
        fprintf (stderr, " error at order_add - %s\n", ifsc_get_last_errmsg(handle));
      break;
    case IFS_OE_AMEND:
      if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_AMEND, buf, len, &orderid)) == 0) {
        if (trace) fprintf (stderr, " order_amend (orderid %d) sent\n", orderid);
      }  
      else
        fprintf (stderr, " error at order_amend - %s\n", ifsc_get_last_errmsg(handle));
      break;
    case IFS_OE_WITHDRAW:
      if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_WITHDRAW, buf, len, &orderid)) == 0) {
        if (trace) fprintf (stderr, " order_del (orderid %d) sent\n", orderid);
      }  
      else
        fprintf (stderr, " error at order_del - %s\n", ifsc_get_last_errmsg(handle));
      break;
    case IFS_OE_TICKUPTICKDOWN:
      if ((s = ifsc_orderentry(handle, IFS_ACTION_TICKUPTICKDOWN, buf, len, &orderid)) == 0) {
        if (trace) fprintf (stderr, " tickuptickdown (orderid %d) sent\n", orderid);
      }  
      else
        fprintf (stderr, " error at tickuptickdown - %s\n", ifsc_get_last_errmsg(handle));
      break;
    default:
      fprintf (stderr, "Bad entry type %c\n", entrytype);
      return;
  }
  fprintf(stdout, "%d\n", orderid);  
}

/*+++++++++++++++++++++++++++++++++++++++++
The parse_line function looks for the identifier in the options structure and writes the value into its variable.
Using MMTS2 fields.

IFSC_HDL Connection handle.

char *line Single line of the config file. The line should look like: identifier = value.

int line_num Line number of the input file.

char entrytype Entry, amend, withdraw ot tickuptickdown.

This function takes a line and splits into fields of orders, then send that into pgw.
+++++++++++++++++++++++++++++++++++++++++*/
static void order_mmts2(IFSC_HDL handle, char *line, int line_num, char entrytype)
{
  char * cp;
  int i, len, s, orderid;
  char ptr_buf[0x8000];
  char buf[1024];
  int fieldnum;
  fixReal real;
  
  switch (entrytype) {
    case IFS_OE_ENTRY:
      fieldnum = FN_ENTRY_MMTS2;
      break;
    case IFS_OE_AMEND:
      fieldnum = FN_AMEND_MMTS2;
      break;
    case IFS_OE_WITHDRAW:
      fieldnum = FN_WITHDRAW_MMTS2;
      break;
	case IFS_OE_TICKUPTICKDOWN:
      fieldnum = FN_TICK_MMTS2 ;
      break;
    default:
      fprintf (stderr, "Bad entry type %c\n", entrytype);
      return;
  }
  // count number of fields
  cp = strchr(line, *SEP);
  for (i=1; i<fieldnum && cp && *cp+1; i++, cp++)
    cp = strchr(cp, *SEP);

  if (i<fieldnum){
    fprintf (stderr, "Missing field from line %d\n", line_num);
    return;
  }

  switch (entrytype) {
    case IFS_OE_ENTRY:
      len = ifs_set_ids(buf, strtok(line, SEP));                                  // trdaccid
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //buysell
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //OrderType
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //Duration
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //PurgeOnLogogg
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //AllowSoftQtyLimit
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //AllowSoftPriceLimit
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //PositionType
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    //IsPrivate
      len += ifs_set_string(buf + len, IFS_BOARDID_LEN, strtok(NULL, SEP));       // boardid
      len += ifs_set_string(buf + len, IFS_SEC_CODE_LEN, strtok(NULL, SEP));      // SecId
    
      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // price - OPTIONAL
    
      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // yield - OPTIONAL
    
      len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                     // qty

      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // visibleqty - OPTIONAL
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);

      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));     // brokerref
      
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // exptime.date - OPTIONAL
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                     // exptime.time - OPTIONAL

      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // triggerprice
    
      len += ifs_set_string(buf + len, IFS_NAME_LEN, strtok(NULL, SEP));          // traderef

      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // MinfillQty - OPTIONAL
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);

      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));     // internal ref
      break;
    case IFS_OE_AMEND:
      len = ifs_set_string(buf, IFS_ORDERNO_LEN, strtok(line, SEP));              // OrdNo
      
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                   // OrdNoSpeedIdx
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      len += ifs_set_ids(buf + len, strtok(NULL, SEP));                           // TrdAcc
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    // Duration
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    // AllowSoftQtyLimit
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    // AllowSoftPriceLimit
      
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_enum(buf + len, atoi(cp));                                 // PositionType
      else
        len += ifs_set_enum(buf + len, NOT_DEFINED);

      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // Price

      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // Yield
      
      len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                     // Qty
      
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // VisibleQty
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));     // BrokerRef

      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // ExpTime.date
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // ExpTime.time
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // TriggerPrice
      
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // MinFillQty
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      len += ifs_set_string(buf + len, IFS_NAME_LEN, strtok(NULL, SEP));          // TradeRef
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));     // IntRef
      break;
      
    case IFS_OE_WITHDRAW:
      len = ifs_set_string(buf, IFS_ORDERNO_LEN, strtok(line, SEP));              // OrdNo

      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                   // OrdNoSpeedIdx
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      len += ifs_set_ids(buf + len, strtok(NULL, SEP));                           // TrdAccId

      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_enum(buf + len, atoi(cp));                                 // BuySell
      else
        len += ifs_set_enum(buf + len, NOT_DEFINED);
        
      len += ifs_set_string(buf + len, IFS_BOARDID_LEN, strtok(NULL, SEP));       // BoardId
      len += ifs_set_ids(buf + len, strtok(NULL, SEP));                           // Instrid
      len += ifs_set_string(buf + len, IFS_SEC_CODE_LEN, strtok(NULL, SEP));      // SecCode

      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // Price
        
      real.value = atof(cp = strtok(NULL, SEP));
      real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
      len += ifs_set_fix_real(buf + len, real);                                   // Yield

      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));     // BrokerRef
      len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                    // OpCode

      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_enum(buf + len, atoi(cp));                                 // PopCode
      else
        len += ifs_set_enum(buf + len, NOT_DEFINED);
        
      len += ifs_set_string(buf + len, IFS_ORDERNO_LEN, strtok(NULL, SEP));        // MultiLegOrdNo

      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                   // MultioLegSpeedIdx
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      len += ifs_set_ids(buf + len, strtok(NULL, SEP));                           // UserId
      len += ifs_set_ids(buf + len, strtok(NULL, SEP));                           // FirmId

      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // OrderDate.date
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(strtok(NULL, SEP)));                   // OrderDate.time
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_enum(buf + len, atoi(cp));                                 // MarketMaker
      else
        len += ifs_set_enum(buf + len, NOT_DEFINED);
        
      len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));     // IntRef
      break;
   case IFS_OE_TICKUPTICKDOWN:
	   len = ifs_set_string(buf, IFS_ORDERNO_LEN, strtok(line, SEP));              // OrdNo
      
      if (!ifs_is_empty_str(cp = strtok(NULL, SEP), IFS_MSG_LEN))
        len += ifs_set_int(buf + len, atoi(cp));                                  // OrdNoSpeedIdx
      else
        len += ifs_set_int(buf + len, NOT_DEFINED);
        len += ifs_set_ids(buf + len, strtok(NULL, SEP));                         // TrdAcc
		len += ifs_set_enum(buf + len, atoi(strtok(NULL, SEP)));                  //buysell
		len += ifs_set_string(buf + len, IFS_BOARDID_LEN, strtok(NULL, SEP));     // BoardId
        len += ifs_set_string(buf + len, IFS_SEC_CODE_LEN, strtok(NULL, SEP));    // SecId	    
		real.value = atof(cp = strtok(NULL, SEP));
        real.decimals = ifs_is_empty_str(cp, IFS_MSG_LEN) ? NOT_DEFINED : 0;
        len += ifs_set_fix_real(buf + len, real);                                 // nopricetick 
	    len += ifs_set_ids(buf + len, strtok(NULL, SEP));                         // UserId
		len += ifs_set_ids(buf + len, strtok(NULL, SEP));                         // FirmId
		len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));   // BrokerRef
		len += ifs_set_string(buf + len, IFS_BROKERREF_LEN, strtok(NULL, SEP));   // internalref
	break;
    default:
      fprintf (stderr, "Bad entry type %c\n", entrytype);
      return;
  }

  if (trace) {
    print_message(ptr_buf, buf, len);
    fprintf(stderr, ">%s<\n",ptr_buf);
  }  

  switch (entrytype) {
    case IFS_OE_ENTRY:
      if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_ADD, buf, len, &orderid)) == 0) {
        if (trace) fprintf (stderr, " order_add (orderid %d) sent\n", orderid);
      } 
      else
        fprintf (stderr, " error at order_add - %s\n", ifsc_get_last_errmsg(handle));
      break;
    case IFS_OE_AMEND:
      if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_AMEND, buf, len, &orderid)) == 0) {
        if (trace) fprintf (stderr, " order_amend (orderid %d) sent\n", orderid);
      }  
      else
        fprintf (stderr, " error at order_amend - %s\n", ifsc_get_last_errmsg(handle));
      break;
    case IFS_OE_WITHDRAW:
      if ((s = ifsc_orderentry(handle, IFS_ACTION_ORDER_WITHDRAW, buf, len, &orderid)) == 0) {
        if (trace) fprintf (stderr, " order_del (orderid %d) sent\n", orderid);
      }  
      else
        fprintf (stderr, " error at order_del - %s\n", ifsc_get_last_errmsg(handle));
      break;
	case IFS_OE_TICKUPTICKDOWN:
      if ((s = ifsc_orderentry(handle, IFS_ACTION_TICKUPTICKDOWN, buf, len, &orderid)) == 0) {
        if (trace) fprintf (stderr, " tickuptickdown (orderid %d) sent\n", orderid);
      }  
      else
        fprintf (stderr, " error at tickuptickdown - %s\n", ifsc_get_last_errmsg(handle));
      break;
    default:
      fprintf (stderr, "Bad entry type %c\n", entrytype);
      return;
  }
  fprintf(stdout, "%d\n", orderid);  
}

/*+++++++++++++++++++++++++++++++++++++++++
This function reads a line from a file into a variable and returns the address of that variable.

static char * get_line A pointer to the line has been read from the config file.

FILE * file File pointer.

char * buffer Buffer area.

int length Length of the buffer.

Those line whose first character is # returned as zero length line. 
+++++++++++++++++++++++++++++++++++++++++*/
static char * get_line (FILE * file, char * buffer, int length)
{
  int i;
  static iseof = 0;

  if (iseof) return NULL;

  for (i = 0; i < length - 1; i++)
    {
      buffer[i] = getc (file);
      if (buffer[i] == 0xa || buffer[i] == EOF)
        break;
    }
  if (i == length - 1)
    fprintf (stderr, "%s -- line too long\n", buffer);

  if (buffer[i] == EOF)
    iseof = 1;
  buffer[i] = '\0';
  if (buffer[0] == '#')
    buffer[0] = '\0';
  return buffer; 
}

/*+++++++++++++++++++++++++++++++++++++++++
Socket based communication interface client. Clients can connect into this server to 
send orderentry request.

int main Tradition.

int argc Number of command line parameters.

char *argv[] Points to an array that contains the command line parameters.
+++++++++++++++++++++++++++++++++++++++++*/
int main (int argc, char *argv[])
{
  int i;
  int tradeid;
  int pid;
  int mmts_type;
  char * recvb;
  int orderid;
  char * host = NULL;
  char * service = NULL, * service1 = NULL, * service2 = NULL;
  char * user = NULL;
  char * pwd = NULL;
  char * order_inp = NULL;
  char entrytype;
  FILE *f;
  char buffer[ORDER_BUF_SIZE];
  IFSC_HDL handle;
  int mmts_mode = 0;

  host = getenv("IFSHOST");
  service1 = getenv("IFSSERVICE");
  service2 = getenv("IFS2SERVICE");
  user = getenv("IFSUSER");
  pwd = getenv("IFSPWD");

  if (argc == 1) usage();
  for (i = 1; i < argc; i++){
    if (argv[i][0] == '-'){
      switch(argv[i][1]){
        case 'e':
          entrytype = *(argv[i] + 2); 
          break;
        case 't': 
          trace = 1; 
          break;
        case 'h':
          host = argv[i]+2;
          break;
        case 's':
          service = argv[i]+2;
          break;
        case 'u':
          user = argv[i]+2;
          break;
        case 'p':
          pwd = argv[i]+2;
          break;
        case 'f':
          order_inp = argv[i]+2;
          break;
        case '1':
          mmts_mode = 1;
          break;
        case '2':
          mmts_mode = 2;
          break;
        case '?':
        case '-':
          usage();
          break;
      }
    } 
  }

  if (trace) fprintf (stderr, "%s Compiled: %s %s\n", Id, __DATE__, __TIME__);

  if (!mmts_mode) {
    fprintf(stderr, "The one of -1 (MMTS1) or -2 (MMTS2) should be set!");
    usage();
  }

  if (mmts_mode == 2 && service2)
    service = service2;
  else
    service = service1;

  if (!host) {
    fprintf(stderr, "The host is not defined!\n");
    usage();
  }

  if (!service) {
    fprintf(stderr, "The service is not defined!\n");
    usage();
  }

  if (!user) {
    fprintf(stderr, "The user is not defined!\n");
    usage();
  }

  if (!pwd) {
    fprintf(stderr, "The pwd is not defined!\n");
    usage();
  }
  
  if (ifsc_create(&handle, host, service)) {
    fprintf(stderr, "Error at creation: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }

  if (trace)
    ifsc_set_trace(handle, IFS_SWITCH_ON);

  if (ifsc_connect(handle, user, pwd, &tradeid, &pid, &mmts_type)) {
    fprintf(stderr, "Error at connection: %s\n", ifsc_get_last_errmsg(handle));
    exit(1);
  }
  
  if (mmts_type != mmts_mode) {
    fprintf(stderr, "Error: mmts type of server connected to is differ from given in command line!");
    exit(1);
  }

  switch (entrytype) {
    case IFS_OE_ENTRY:
    case IFS_OE_AMEND:
    case IFS_OE_WITHDRAW:
    case IFS_OE_TICKUPTICKDOWN:
      break;
    default:
      fprintf(stderr, "Invalid entry type <E/A/W/T>\n");
      usage();
  }

  if (order_inp)
    f = fopen (order_inp, "r");
  else
    f = stdin;  

  if (!f){
    fprintf (stderr, "%s\n", strerror(errno));
    exit(-1);
  }

  for (i=1; get_line (f, buffer, ORDER_BUF_SIZE); i++) 
  {
    if (buffer[0] != '\0') 
      if (mmts_mode == 1)
        order_mmts1(handle, buffer, i, entrytype);
      else
        order_mmts2(handle, buffer, i, entrytype);
  }  

  fclose (f);

  return ifsc_disconnect(handle);
}
